from __future__ import annotations

import logging

from tano_common.logging import bind_context as _bind_context
from tano_common.logging import configure_logging as _configure_logging


def setup_logging(level: int = logging.INFO) -> None:
    _configure_logging(level=level)


def bind_context(**kwargs):
    _bind_context(**kwargs)
    return kwargs
