from fastapi import APIRouter

from app.api.v1.endpoints import firewall, scan, pii, credits, api_keys

router = APIRouter()

router.include_router(firewall.router, prefix="/firewall")
router.include_router(scan.router, prefix="/scan")
router.include_router(pii.router, prefix="/pii-redaction")
router.include_router(credits.router, prefix="/credits")
router.include_router(api_keys.router, prefix="/api-keys")
